@@ -1,8 +1,10 @@
 
-.ctmfit <- function(object, parm, mltargs, reparm) {
+.ctmfit <- function(object, parm, mltargs, reparm, 
+                    min_update = length(coef(object)) * 2) {
     
     ctmobject <- object
 
+    ### note: control is never used but expected by partykit::ctree
     function(data, weights, control, ...) {
         mf <- model.frame(data, yxonly = TRUE)
         iy <- data[["yx", type = "index"]]
@@ -32,15 +34,25 @@
             } else {
                 thetastart <- coef(ctmobject, fixed = FALSE)
             }
-            umod <- suppressWarnings(try(update(ctmobject, weights = w, subset = subset, theta = thetastart), silent = TRUE))
-            if (inherits(umod, "try-error") || umod$convergence != 0) {
-                umod <- suppressWarnings(try(update(ctmobject, weights = w, subset = subset), silent = TRUE))
+            ### update parameters when there are more than min_update
+            ### observations
+            if (length(subset) > min_update) {
+                umod <- suppressWarnings(try(update(ctmobject, weights = w, 
+                    subset = subset, theta = thetastart), silent = TRUE))
                 if (inherits(umod, "try-error") || umod$convergence != 0) {
-                    mltargs$weights <- w
-                    ### [1]: no subset allowed here, so used zero weights (see
-                    ### above AND below)!!!
-                    umod <- try(do.call("mlt", mltargs))
+                    umod <- suppressWarnings(try(update(ctmobject, weights = w, subset = subset), 
+                                                  silent = TRUE))
+                    if (inherits(umod, "try-error") || umod$convergence != 0) {
+                        mltargs$weights <- w
+                        ### [1]: no subset allowed here, so used zero weights (see
+                        ### above AND below)!!!
+                        umod <- try(do.call("mlt", mltargs))
+                    }
                 }
+            } else {
+                ### reuse parameters from root node (next if statement)
+                umod <- 0
+                class(umod) <- "try-error"
             }
             if (inherits(umod, "try-error")) {
                 ### we badly need some estimate in each node, even if fitting
@@ -48,8 +60,22 @@
                 if (!estfun) 
                     return(list(coef = thetastart, objfun = NA,
                                 converged = FALSE))
-                return(list(estfun = matrix(0, nrow = nrow(mf), ncol = length(w)),
-                            coef = thetastart, objfun = NA,  converged = FALSE))
+                ### estfun was requested and thus a tree wants to look for
+                ### splits. We cannot update the model, so we simply reuse
+                ### the start parameters. This amounts to treating the
+                ### scores as fix when building the subtree. This was always
+                ### the case in party::ctree (the "model" was once estimated
+                ### in the root node) and Stefan Wager suggested to apply
+                ### this scheme in model-based trees in a discussion 2019-09-06.
+                # mltargs$weights <- w
+                # mltargs$doFit <- FALSE
+                # umod <- try(do.call("mlt", mltargs))
+                umod <- ctmobject
+                umod$weights <- w
+                umod$subset <- subset
+                coef(umod)[names(thetastart)] <- thetastart
+                umod$convergence <- 0L ### this is FAKE but extree would stop
+                                       ### after non-convergence
             }
             ret <- NULL
             if (estfun) {
@@ -78,7 +104,8 @@
     }
 } 
 
-trafotree <- function(object, parm = 1:length(coef(object)), reparm = NULL,
+trafotree <- function(object, parm = 1:length(coef(object)), reparm = NULL, 
+                      min_update = length(coef(object)) * 2, 
                       mltargs = list(maxit = 10000), ...) {
 
     ### we only work with the ctm object
@@ -92,7 +119,9 @@ trafotree <- function(object, parm = 1:length(coef(object)), reparm = NULL,
     mltargs$model <- object
     ### note: weights, offset, cluster etc. are evaluated here !!!
     args <- list(...)
-    args$ytrafo <- .ctmfit(object, parm, mltargs, reparm)
+    args$ytrafo <- .ctmfit(object = object, parm = parm, 
+                           mltargs = mltargs, reparm = reparm, 
+                           min_update = min_update)
     args$update <- TRUE
     ret <- do.call("ctree", args)
     ret$model <- object
@@ -115,7 +144,8 @@ trafotree <- function(object, parm = 1:length(coef(object)), reparm = NULL,
 }
 
 traforest <- function(object, parm = 1:length(coef(object)), reparm = NULL,
-                      mltargs = list(maxit = 10000), update = TRUE, ...) {
+                      update = TRUE, min_update = length(coef(object)) * 2, 
+                      mltargs = list(maxit = 10000), ...) {
 
     if (inherits(object, "mlt")) object <- object$model
     ### this is tricky because parm is only valid
@@ -123,7 +153,9 @@ traforest <- function(object, parm = 1:length(coef(object)), reparm = NULL,
     mltargs$model <- object
     ### note: weights, offset, cluster etc. are evaluated here !!!
     args <- list(...)
-    args$ytrafo <- .ctmfit(object, parm, mltargs, reparm)
+    args$ytrafo <- .ctmfit(object = object, parm = parm, 
+                           mltargs = mltargs, reparm = reparm,
+                           min_update = min_update)
     args$update <- update
     ret <- do.call("cforest", args)
     ret$model <- object
